<?php

declare(strict_types = 1);

namespace Drupal\commerce_otpsp\PluginForm;

use Cheppers\OtpspClient\OtpSimplePayClient;
use Drupal\commerce\Response\NeedsRedirectException;
use Drupal\commerce_otpsp\PaymentDataProvider;
use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

class SimplePayRedirectOffSitePaymentForm extends PaymentOffsiteForm implements ContainerInjectionInterface {

  /**
   * @var \Cheppers\OtpspClient\OtpSimplePayClient
   */
  protected $client;

  /**
   * @var \Drupal\commerce_otpsp\PaymentDataProvider
   */
  protected $paymentDataProvider;

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_otpsp.client'),
      $container->get('commerce_otpsp.payment_data_provider')
    );
  }

  public function __construct(
    OtpSimplePayClient $client,
    PaymentDataProvider $paymentDataProvider
  ) {
    $this->client = $client;
    $this->paymentDataProvider = $paymentDataProvider;
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(
    array $form,
    FormStateInterface $form_state
  ) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->getEntity();
    $payment->save();

    $paymentDataProvider = $this->paymentDataProvider;
    $paymentDataProvider->setUp($payment->id());

    return $this->buildRedirectForm(
      $form,
      $form_state,
      $this->client->getLiveUpdateUrl(),
      $paymentDataProvider->getPreparedData(),
      self::REDIRECT_POST);
  }

  /**
   * {@inheritdoc}
   */
  protected function buildRedirectForm(array $form, FormStateInterface $form_state, $redirect_url, array $data, $redirect_method = self::REDIRECT_GET) {
    if ($redirect_method === static::REDIRECT_GET) {
      $redirect_url = Url::fromUri($redirect_url, ['absolute' => TRUE, 'query' => $data])->toString();

      throw new NeedsRedirectException($redirect_url);
    }

    $form['#attached']['library'][] = 'commerce_payment/offsite_redirect';
    $form['#process'][] = [get_class($this), 'processRedirectForm'];
    $form['#redirect_url'] = $redirect_url;
    $i = 0;

    foreach ($data as $number) {
      foreach ($number as $key => $value) {
        $form[$i++] = [
          '#type' => 'hidden',
          '#value' => $value,
          '#name' => $key,
          '#parents' => [$key],
        ];
      }
    }

    $form['commerce_message'] = [
      '#markup' => '<div class="checkout-help">' . $this->t('Please wait while you are redirected to the payment server. If nothing happens within 10 seconds, please click on the button below.') . '</div>',
      '#weight' => -10,
    ];

    return $form;
  }

}
